<?php
// $Id: template.php,v 1.20 2003/07/08 12:38:08 okazu Exp $
//  ------------------------------------------------------------------------ //
//                XOOPS - PHP Content Management System                      //
//                    Copyright (c) 2000 XOOPS.org                           //
//                       <http://www.xoops.org/>                             //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //
// Author: Kazumi Ono (AKA onokazu)                                          //
// URL: http://www.myweb.ne.jp/, http://www.xoops.org/, http://jp.xoops.org/ //
// Project: The XOOPS Project                                                //
// ------------------------------------------------------------------------- //

if (!defined('SMARTY_DIR')) {
	exit();
}
/**
 * Base class: Smarty template engine
 */
require_once SMARTY_DIR.'Smarty.class.php';

/**
 * Template engine
 *
 * @package		kernel
 * @subpackage	core
 *
 * @author		Kazumi Ono 	<onokazu@xoops.org>
 * @copyright	(c) 2000-2003 The Xoops Project - www.xoops.org
 */
class XoopsTpl extends Smarty
{

	/**
	 * Allow update of template files from the themes/ directory?
     * This should be set to false on an active site to increase performance
	 */
	var $_canUpdateFromFile = false;
  	var $_recentModule = '';

	/**
	 * Constructor
	 **/
	function XoopsTpl()
	{
		global $xoopsConfig;
		$this->Smarty();
		$this->compile_id = null;
    	$this->force_compile = true;
		if ($xoopsConfig['theme_fromfile'] == 1) {
			$this->_canUpdateFromFile = true;
			$this->compile_check = true;
		} else {
			$this->_canUpdateFromFile = false;
			$this->compile_check = false;
		}
		$this->left_delimiter =  '<{';
		$this->right_delimiter =  '}>';
		$this->template_dir = XOOPS_THEME_PATH;
		$this->cache_dir = XOOPS_CACHE_PATH;
		$this->compile_dir = XOOPS_COMPILE_PATH;
		$this->plugins_dir = array(XOOPS_ROOT_PATH.'/class/smarty/plugins');
		$this->default_template_handler_func = 'xoops_template_create';
		$this->use_sub_dirs = false;
		$this->assign(array('xoops_url' => XOOPS_URL, 'xoops_rootpath' => XOOPS_ROOT_PATH, 'xoops_langcode' => _LANGCODE, 'xoops_charset' => _CHARSET, 'xoops_version' => XOOPS_VERSION, 'xoops_upload_url' => XOOPS_UPLOAD_URL));
	}

	/**
	 * Set the directory for templates
     * 
     * @param   string  $dirname    Directory path without a trailing slash
	 **/
	function xoops_setTemplateDir($dirname)
	{
		$this->template_dir = $dirname;
	}

	/**
	 * Get the active template directory
	 * 
	 * @return  string
	 **/
	function xoops_getTemplateDir()
	{
		return $this->template_dir;
	}

	/**
	 * Set debugging mode
	 * 
	 * @param   boolean     $flag
	 **/
	function xoops_setDebugging($flag=false)
	{
		$this->debugging = is_bool($flag) ? $flag : false;
	}

	/**
	 * Set caching
	 * 
	 * @param   integer     $num
	 **/
	function xoops_setCaching($num=0)
	{
		$this->caching = (int)$num;
	}

	/**
	 * Set cache lifetime
	 * 
	 * @param   integer     $num    Cache lifetime
	 **/
	function xoops_setCacheTime($num=0)
	{
		$num = (int)$num;
		if ($num <= 0) {
			$this->caching = 0;
		} else {
			$this->cache_lifetime = $num;
		}
	}

	/**
	 * Set directory for compiled template files
	 * 
	 * @param   string  $dirname    Full directory path without a trailing slash
	 **/
	function xoops_setCompileDir($dirname)
	{
		$this->compile_dir = $dirname;
	}

	/**
	 * Set the directory for cached template files
	 * 
	 * @param   string  $dirname    Full directory path without a trailing slash
	 **/
	function xoops_setCacheDir($dirname)
	{
		$this->cache_dir = $dirname;
	}

	/**
	 * Render output from template data
	 * 
	 * @param   string  $data
	 * @return  string  Rendered output  
	 **/
	function xoops_fetchFromData(&$data)
	{
		$dummyfile = XOOPS_CACHE_PATH.'/dummy_'.time();
		$fp = fopen($dummyfile, 'w');
		fwrite($fp, $data);
		fclose($fp);
		$fetched = $this->fetch('file:'.$dummyfile);
		unlink($dummyfile);
		$this->clear_compiled_tpl('file:'.$dummyfile);
		return $fetched;
	}

	/**
	 * 
	 **/
	function xoops_canUpdateFromFile()
	{
		return $this->_canUpdateFromFile;
	}
  
  /**
   * called for included templates
   *
   * @param string $_smarty_include_tpl_file
   * @param string $_smarty_include_vars
   */
  function _smarty_include($params)
  {
    $_include_name_parts = explode(':', $params['smarty_include_tpl_file'], 2);
    if ((count($_include_name_parts) == 2) && ($_include_name_parts[0] == "db") && ($this->_recentModule)) {
      if ($GLOBALS['xoopsConfig']['template_set'] != 'default' ) {
         $_theme_name = '_';
      } else {
         $_theme_name = $GLOBALS['xoopsConfig']['theme_set'];
      }
      $params['smarty_include_tpl_file'] = 'db:'.$_theme_name."!".$this->_recentModule."!".$_include_name_parts[1];
    }
//  echo "{$params['smarty_include_tpl_file']}<br>";
    parent::_smarty_include($params);
  }
  
  /**
   * parse out the type and name from the resource
   *   Original by OreXOOPS by Marijuana;
   * @param string $resource_base_path
   * @param string $resource_name
   * @param string $resource_type
   * @param string $resource_name
   * @return boolean
   */
  function _parse_resource_name(&$params)
  {
   // echo "{$params['resource_name']}<br>";
    $orig_prams = $params;
    // split tpl_path by the first colon
    $_resource_name_parts = explode(':', $params['resource_name'], 2);
    if (count($_resource_name_parts) == 1) {
      // no resource type given
      $params['resource_type'] = $this->default_resource_type;
      $params['resource_name'] = $_resource_name_parts[0];
    } else {
      if(strlen($_resource_name_parts[0]) == 1) {
      // 1 char is not resource type, but part of filepath
        $params['resource_type'] = $this->default_resource_type;
        $params['resource_name'] = $params['resource_name'];
      } else {
        $params['resource_type'] = $_resource_name_parts[0];
        $params['resource_name'] = $_resource_name_parts[1];
      }
    }
    //-----------------------------------------------ɲ
    //echo $params['resource_name'];
    if ( strpos($params['resource_name'],'!') === false && $params['resource_type'] == 'db') {
      if ($GLOBALS['xoopsConfig']['template_set'] != 'default' ) {
         return parent:: _parse_resource_name($orig_prams);
      }
      /** * ƥ⥸塼롢ƥץ졼⥤󥯥롼¾ */
      $_temp = explode('_', $params['resource_name']);
      $_module_name = $_temp[0];
      $_mod_r = XOOPS_THEME_PATH.'/'.$GLOBALS['xoopsConfig']['theme_set'].'/templates/'.$_module_name.'/';
      $_mod_b = $_mod_r.'blocks/';
      if ( ( file_exists($_mod_r.$params['resource_name']) || file_exists($_mod_b.$params['resource_name']) ) && $GLOBALS['xoopsConfig']['theme_set'] != 'default' ) {
        /** * ơޤǥեȰʳǥơޥե˥ƥץ졼Ȥ */
        $params['resource_base_path'] = array(
          XOOPS_THEME_PATH.'/'.$GLOBALS['xoopsConfig']['theme_set'].'/templates/'.$_module_name.'/',
          XOOPS_THEME_PATH.'/'.$GLOBALS['xoopsConfig']['theme_set'].'/templates/'.$_module_name.'/blocks/'
        );
      } else {
        /** * ⥸塼Υƥץ졼ȥե򥻥å */
        $params['resource_base_path'] = array(
          XOOPS_ROOT_PATH.'/modules/'.$_module_name.'/templates/',
          XOOPS_ROOT_PATH.'/modules/'.$_module_name.'/templates/blocks'
        );
      }
      $this->_recentModule = "";
      /** * եɹ褦˥å */
      $params['resource_type'] = 'file';
    } elseif ( $params['resource_type'] == 'db' && strpos($params['resource_name'],'!') !== false ) {
      /** * ơ!ǥ쥯ȥ!ե̾Ƥ   */
      $_temp = explode('!', $params['resource_name']);
      if ( $_temp[1] == '____' ) {
        $_temp[1] = 'system';
      }
      $_module_name = $_temp[1];
      $_mod_r = XOOPS_THEME_PATH.'/'.$_temp[0].'/templates/'.$_module_name.'/';
      $_mod_b = $_mod_r.'blocks/';
      if ($GLOBALS['xoopsConfig']['template_set'] != 'default' ) {
	     $this->_recentModule = $_module_name;
         return parent:: _parse_resource_name($orig_prams);
      }
      if ( ( file_exists($_mod_r.$_temp[2]) || file_exists($_mod_b.$_temp[2]) ) && $_temp[0] != 'default' ) {
        /** * ơޤǥեȰʳǥơޥե˥ƥץ졼Ȥ */
        $params['resource_base_path'] = array(
          XOOPS_THEME_PATH.'/'.$_temp[0].'/templates/'.$_module_name.'/',
          XOOPS_THEME_PATH.'/'.$_temp[0].'/templates/'.$_module_name.'/blocks/'
        );
      } else {
        /** * ⥸塼Υƥץ졼ȥե򥻥å */
        $params['resource_base_path'] = array(
          XOOPS_ROOT_PATH.'/modules/'.$_module_name.'/templates/',
          XOOPS_ROOT_PATH.'/modules/'.$_module_name.'/templates/blocks/'
        );
      }
      $this->_recentModule = $_module_name;
      /** * եɹ褦˥å */
      $params['resource_type'] = 'file';
      $params['resource_name'] = $_temp[2];
    }
    //-----------------------------------------------
    if ($params['resource_type'] == 'file') {
      if (!preg_match('/^([\/\\\\]|[a-zA-Z]:[\/\\\\])/', $params['resource_name'])) {
        // relative pathname to $params['resource_base_path']
        // use the first directory where the file is found
        foreach ((array)$params['resource_base_path'] as $_curr_path) {
          $_fullpath = $_curr_path . DIRECTORY_SEPARATOR . $params['resource_name'];
          if (file_exists($_fullpath) && is_file($_fullpath)) {
            $params['resource_name'] = $_fullpath;
            return true;
          }
          // didn't find the file, try include_path
          $_params = array('file_path' => $_fullpath);
          require_once(SMARTY_DIR . 'core' . DIRECTORY_SEPARATOR . 'core.get_include_path.php');
          if(smarty_core_get_include_path($_params, $this)) {
            $params['resource_name'] = $_params['new_file_path'];
            return true;
          }
        }
        return false;
      } else {
        /* absolute path */
        return file_exists($params['resource_name']);
      }
    } elseif (empty($this->_plugins['resource'][$params['resource_type']])) {
      $_params = array('type' => $params['resource_type']);
      require_once(SMARTY_DIR . 'core' . DIRECTORY_SEPARATOR . 'core.load_resource_plugin.php');
      smarty_core_load_resource_plugin($_params, $this);
    }
    return true;
  }
  
}

/**
 * Smarty default template handler function
 * 
 * @param $resource_type
 * @param $resource_name
 * @param $template_source
 * @param $template_timestamp
 * @param $smarty_obj
 * @return  bool
 **/
function xoops_template_create ($resource_type, $resource_name, &$template_source, &$template_timestamp, &$smarty_obj)
{
//XOOPSѤΥ꥽ϥɥºݤˤϤۤȤɤresource.db.phpץ饰ǽƤ롣
//äƤǽƤΤϡƥץ졼ȥåȤǥǥեȰʳѤƤơ
//Υƥץ졼ȥå˻Υƥץ졼ȥե뤬¸ߤʤȤˤΤߡ
	if ( $resource_type == 'db' ) {
		$file_handler =& xoops_gethandler('tplfile');
//hacked by nobunobu start
//		$tpl =& $file_handler->find('default', null, null, null, $resource_name, true);
	   	$mod_name = '____';
	    if (preg_match("/([^\!]*?)\!([^\!]*?)\!(.*)/",$resource_name,$matches) ){
	    	$resource_name = $matches[3];
	    	$mod_name = $matches[2];
	    } else if (preg_match("/([^\!]*?)\!(.*)/",$resource_name,$matches) ){
	    	$resource_name = $matches[2];
	    	$mod_name = $matches[1];
	    }
	    if ($mod_name == '____') {
	    	$mod_name = null;
	    }
		$tpl =& $file_handler->find('default', null, null, $mod_name, $resource_name, true);
//hacked by nobunobu end
		if (count($tpl) > 0 && is_object($tpl[0])) {
			$template_source = $tpl[0]->getSource();
			$template_timestamp = $tpl[0]->getLastModified();
			return true;
		}
	} else {
	}
	return false;
}

/**
 * function to update compiled template file in templates_c folder
 * 
 * @param   string  $tpl_id
 * @param   boolean $clear_old
 * @return  boolean
 **/
function xoops_template_touch($tpl_id, $clear_old = true)
{
	$tpl = new XoopsTpl();
	$tpl->force_compile = true;
	$tplfile_handler =& xoops_gethandler('tplfile');
	$tplfile =& $tplfile_handler->get($tpl_id);
	$config_handler =& xoops_gethandler('config');
	$criteria = new CriteriaCompo();
	$criteria->add(new Criteria('conf_modid', 0));
	$criteria->add(new Criteria('conf_catid', 1));
	$criteria->add(new Criteria('conf_name', 'template_set'));
	$configs =& $config_handler->getConfigs($criteria);
	foreach($configs as $config) {
		$cur_template_set =$config->getVar('conf_value');
	}
	$new_template_set = $GLOBALS['xoopsConfig']['template_set'];
	if ( is_object($tplfile) ) {
		$file = $tplfile->getVar('tpl_file');
//hacked by nobunobu start
		$mod = $tplfile->getVar('tpl_module');
		if( $cur_template_set != 'default' ) {
			$old_theme = "_!";
		} else {
			$old_theme = trim($GLOBALS['xoopsConfig']['theme_set']."!");
		}
		if( $new_template_set != 'default' ) {
			$theme = "_!";
		} else {
			$theme = trim($GLOBALS['xoopsConfig']['theme_set']."!");
		}
		$GLOBALS['xoopsConfig']['template_set'] = $cur_template_set;
		if ($clear_old) {
//			$tpl->clear_cache('db:'.$file);
//			$tpl->clear_compiled_tpl('db:'.$file);
			$tpl->clear_cache('db:'.$old_theme.$mod.'!'.$file);
			$tpl->clear_compiled_tpl('db:'.$old_theme.$mod.'!'.$file);
		}
		$GLOBALS['xoopsConfig']['template_set'] = $new_template_set;
//		$tpl->fetch('db:'.$file);
		$tpl->fetch('db:'.$theme.$mod.'!'.$file);
//hacked by nobunobu end
		return true;
	}
	return false;
}

/**
 * Clear the module cache
 * 
 * @param   int $mid    Module ID
 * @return 
 **/
function xoops_template_clear_module_cache($mid)
{
	$block_arr =& XoopsBlock::getByModule($mid);
	$count = count($block_arr);
	if ($count > 0) {
		$xoopsTpl = new XoopsTpl();	
		$xoopsTpl->xoops_setCaching(2);
		for ($i = 0; $i < $count; $i++) {
			if ($block_arr[$i]->getVar('template') != '') {
				$xoopsTpl->clear_cache('db:'.$block_arr[$i]->getVar('template'));
			}
		}
	}
}
?>